@@ -4,7 +4,7 @@ import logging
 import socket
 from collections import deque
 from copy import copy
-from typing import Any, Optional
+from typing import Any, Callable, List, Optional, Set, Tuple
 
 from .. import introspection as intr
 from ..auth import Authenticator, AuthExternal
@@ -18,7 +18,7 @@ from ..constants import (
 )
 from ..errors import AuthError
 from ..message import Message
-from ..message_bus import BaseMessageBus
+from ..message_bus import BaseMessageBus, _block_unexpected_reply
 from ..service import ServiceInterface
 from .message_reader import build_message_reader
 from .proxy_object import ProxyObject
@@ -26,6 +26,19 @@ from .proxy_object import ProxyObject
 NO_REPLY_EXPECTED_VALUE = MessageFlag.NO_REPLY_EXPECTED.value
 
 
+def _generate_hello_serialized(next_serial: int) -> bytes:
+    return Message(
+        destination="org.freedesktop.DBus",
+        path="/org/freedesktop/DBus",
+        interface="org.freedesktop.DBus",
+        member="Hello",
+        serial=next_serial,
+    )._marshall(False)
+
+
+HELLO_1_SERIALIZED = _generate_hello_serialized(1)
+
+
 def _future_set_exception(fut: asyncio.Future, exc: Exception) -> None:
     if fut is not None and not fut.done():
         fut.set_exception(exc)
@@ -37,28 +50,38 @@ def _future_set_result(fut: asyncio.Future, result: Any) -> None:
 
 
 class _MessageWriter:
+    """A class to handle writing messages to the message bus."""
+
     def __init__(self, bus: "MessageBus") -> None:
-        self.messages = deque()
+        """A class to handle writing messages to the message bus."""
+        self.messages: deque[
+            Tuple[bytearray, Optional[List[int]], Optional[asyncio.Future]]
+        ] = deque()
         self.negotiate_unix_fd = bus._negotiate_unix_fd
         self.bus = bus
         self.sock = bus._sock
         self.loop = bus._loop
-        self.buf = None
+        self.buf: Optional[memoryview] = None
         self.fd = bus._fd
         self.offset = 0
-        self.unix_fds = None
+        self.unix_fds: Optional[List[int]] = None
         self.fut: Optional[asyncio.Future] = None
 
     def write_callback(self, remove_writer: bool = True) -> None:
+        """The callback to write messages to the message bus."""
+        sock = self.sock
         try:
             while True:
                 if self.buf is None:
+                    # If there is no buffer, get the next message
                     if not self.messages:
                         # nothing more to write
                         if remove_writer:
                             self.loop.remove_writer(self.fd)
                         return
-                    buf, unix_fds, fut = self.messages.pop()
+
+                    # Get the next message
+                    buf, unix_fds, fut = self.messages.popleft()
                     self.unix_fds = unix_fds
                     self.buf = memoryview(buf)
                     self.offset = 0
@@ -72,18 +95,18 @@ class _MessageWriter:
                             array.array("i", self.unix_fds),
                         )
                     ]
-                    self.offset += self.sock.sendmsg([self.buf[self.offset :]], ancdata)
+                    self.offset += sock.sendmsg([self.buf[self.offset :]], ancdata)
                     self.unix_fds = None
                 else:
-                    self.offset += self.sock.send(self.buf[self.offset :])
+                    self.offset += sock.send(self.buf[self.offset :])
 
-                if self.offset >= len(self.buf):
-                    # finished writing
-                    self.buf = None
-                    _future_set_result(self.fut, None)
-                else:
+                if self.offset < len(self.buf):
                     # wait for writable
                     return
+
+                # finished writing
+                self.buf = None
+                _future_set_result(self.fut, None)
         except Exception as e:
             if self.bus._user_disconnect:
                 _future_set_result(self.fut, None)
@@ -91,11 +114,15 @@ class _MessageWriter:
                 _future_set_exception(self.fut, e)
             self.bus._finalize(e)
 
-    def buffer_message(self, msg: Message, future=None) -> None:
+    def buffer_message(
+        self, msg: Message, future: Optional[asyncio.Future] = None
+    ) -> None:
+        """Buffer a message to be sent later."""
+        unix_fds = msg.unix_fds
         self.messages.append(
             (
                 msg._marshall(self.negotiate_unix_fd),
-                copy(msg.unix_fds),
+                copy(unix_fds) if unix_fds else None,
                 future,
             )
         )
@@ -104,10 +131,14 @@ class _MessageWriter:
         """Call the write callback without removing the writer."""
         self.write_callback(remove_writer=False)
 
-    def schedule_write(self, msg: Optional[Message] = None, future=None) -> None:
+    def schedule_write(
+        self, msg: Optional[Message] = None, future: Optional[asyncio.Future] = None
+    ) -> None:
+        """Schedule a message to be written."""
         queue_is_empty = not self.messages
         if msg is not None:
             self.buffer_message(msg, future)
+
         if self.bus.unique_name:
             # Optimization: try to send now if the queue
             # is empty. With bleak this usually means we
@@ -115,6 +146,7 @@ class _MessageWriter:
             # is a huge improvement in latency.
             if queue_is_empty:
                 self._write_without_remove_writer()
+
             if (
                 self.buf is not None
                 or self.messages
@@ -154,7 +186,7 @@ class MessageBus(BaseMessageBus):
     :vartype connected: bool
     """
 
-    __slots__ = ("_loop", "_auth", "_writer", "_disconnect_future")
+    __slots__ = ("_loop", "_auth", "_writer", "_disconnect_future", "_pending_futures")
 
     def __init__(
         self,
@@ -174,6 +206,7 @@ class MessageBus(BaseMessageBus):
             self._auth = auth
 
         self._disconnect_future = self._loop.create_future()
+        self._pending_futures: Set[asyncio.Future] = set()
 
     async def connect(self) -> "MessageBus":
         """Connect this message bus to the DBus daemon.
@@ -214,18 +247,14 @@ class MessageBus(BaseMessageBus):
                 self.disconnect()
                 self._finalize(err)
 
-        hello_msg = Message(
-            destination="org.freedesktop.DBus",
-            path="/org/freedesktop/DBus",
-            interface="org.freedesktop.DBus",
-            member="Hello",
-            serial=self.next_serial(),
-        )
-
-        self._method_return_handlers[hello_msg.serial] = on_hello
-        self._stream.write(hello_msg._marshall(False))
+        next_serial = self.next_serial()
+        self._method_return_handlers[next_serial] = on_hello
+        if next_serial == 1:
+            serialized = HELLO_1_SERIALIZED
+        else:
+            serialized = _generate_hello_serialized(next_serial)
+        self._stream.write(serialized)
         self._stream.flush()
-
         return await future
 
     async def introspect(
@@ -412,12 +441,33 @@ class MessageBus(BaseMessageBus):
         if not asyncio.iscoroutinefunction(method.fn):
             return super()._make_method_handler(interface, method)
 
-        def _coro_method_handler(msg, send_reply):
-            def done(fut):
+        negotiate_unix_fd = self._negotiate_unix_fd
+        msg_body_to_args = ServiceInterface._msg_body_to_args
+        fn_result_to_body = ServiceInterface._fn_result_to_body
+
+        def _coroutine_method_handler(
+            msg: Message, send_reply: Callable[[Message], None]
+        ) -> None:
+            """A coroutine method handler."""
+            args = msg_body_to_args(msg) if msg.unix_fds else msg.body
+            fut = asyncio.ensure_future(method.fn(interface, *args))
+            # Hold a strong reference to the future to ensure
+            # it is not garbage collected before it is done.
+            self._pending_futures.add(fut)
+            if (
+                send_reply is _block_unexpected_reply
+                or msg.flags.value & NO_REPLY_EXPECTED_VALUE
+            ):
+                fut.add_done_callback(self._pending_futures.discard)
+                return
+
+            # We only create the closure function if we are actually going to reply
+            def _done(fut: asyncio.Future) -> None:
+                """The callback for when the method is done."""
                 with send_reply:
                     result = fut.result()
-                    body, unix_fds = ServiceInterface._fn_result_to_body(
-                        result, method.out_signature_tree
+                    body, unix_fds = fn_result_to_body(
+                        result, method.out_signature_tree, replace_fds=negotiate_unix_fd
                     )
                     send_reply(
                         Message.new_method_return(
@@ -425,11 +475,11 @@ class MessageBus(BaseMessageBus):
                         )
                     )
 
-            args = ServiceInterface._msg_body_to_args(msg)
-            fut = asyncio.ensure_future(method.fn(interface, *args))
-            fut.add_done_callback(done)
+            fut.add_done_callback(_done)
+            # Discard the future only after running the done callback
+            fut.add_done_callback(self._pending_futures.discard)
 
-        return _coro_method_handler
+        return _coroutine_method_handler
 
     async def _auth_readline(self) -> str:
         buf = b""
